package de.opalpiraten.scenepainter.gui; 
import java.awt.*; 
import java.io.*;
import javax.swing.*;
import java.util.prefs.*;
import java.awt.event.*;
import java.util.*;
import de.opalpiraten.scenepainter.sys.*;
import de.opalpiraten.scenepainter.gui.render.*;


/**
 * Allows to reconfigure file pathes and stores them in the preferences.
 * Note: This class is an ad-hoc, self contained solution and will be replaced, 
 * if more functionality is needed (e.g. tools wanting path settings).
 */
public class PathSettings extends JInternalFrame implements ActionListener {

  /**
   * Apply changes
   */
  private JButton apply = new JButton(Util.createIcon("toolbarButtonGraphics/general/Save16.gif",""));
  
  /**
   * From where to load nenya
   */
  private JTextField nenyadir = new JTextField(30);
  
  /**
   * Where to save PNG files to (if not in project)
   */
  private JTextField exportdir = new JTextField(30);
  
  /**
   * Where to save elements (if not in project)
   */
  private JTextField elementdir = new JTextField(30);    
  
  /**
   * For pluging into the config menu
   */
  private static JMenuItem item;
  
  /**
   * Construct the window
   */
  private PathSettings() {
    super(Dict.get("gui.pathsettings.menuitem"),false,true,false,false);
    JPanel content = new JPanel();
    JPanel list = new JPanel();
    
    Preferences cfg = Preferences.userNodeForPackage(Core.class);
    nenyadir.setText(cfg.get("nenyabase",null));
    exportdir.setText(cfg.get("centralexportdir",null));
    elementdir.setText(cfg.get("centralelementdir",null));
    
    
    GridBagLayout gbl = new GridBagLayout();
    setLayout(gbl);
    GridBagConstraints gbc = new GridBagConstraints();
    content.setLayout(gbl);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=1.0;
    gbc.weighty=1.0;
    gbc.insets = new Insets(2,2,2,2);
    JLabel lbl0 = new JLabel(Dict.get("gui.pathsettings.nenyadir"));
    gbl.setConstraints(lbl0,gbc);
    content.add(lbl0);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=1;
    gbc.gridy=0;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(2,2,2,2);
    gbl.setConstraints(nenyadir,gbc);
    content.add(nenyadir);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=1;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(2,2,2,2);
    JLabel lbl1 = new JLabel(Dict.get("gui.pathsettings.exportdir"));
    gbl.setConstraints(lbl1,gbc);
    content.add(lbl1);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=1;
    gbc.gridy=1;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(2,2,2,2);
    gbl.setConstraints(exportdir,gbc);
    content.add(exportdir);
    
    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=0;
    gbc.gridy=2;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(2,2,2,2);
    JLabel lbl2 = new JLabel(Dict.get("gui.pathsettings.elementdir"));
    gbl.setConstraints(lbl2,gbc);
    content.add(lbl2);

    gbc.anchor=GridBagConstraints.NORTHWEST;
    gbc.fill=GridBagConstraints.BOTH;
    gbc.gridx=1;
    gbc.gridy=2;
    gbc.gridwidth=1; 
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(2,2,2,2);
    gbl.setConstraints(elementdir,gbc);
    content.add(elementdir);
    
    gbc.anchor=GridBagConstraints.NORTHEAST;
    gbc.fill=GridBagConstraints.NONE;
    gbc.gridx=1;
    gbc.gridy=3;
    gbc.gridwidth=2; 
    gbc.gridheight=1;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.insets = new Insets(2,2,2,2);
    gbl.setConstraints(apply,gbc);
    content.add(apply);
    
    setContentPane(content);
    apply.addActionListener(this);
    elementdir.addActionListener(this);
    nenyadir.addActionListener(this);
    exportdir.addActionListener(this);
    pack();
  }
  
  /**
   * For bootstrapping
   * @param b dont care
   */
  private PathSettings(int b) {}
  

  public void actionPerformed(ActionEvent e) {
    
    if (e.getSource()==item) {
      PathSettings ps = new PathSettings();
      Core.workspace.add(ps);
      ps.setLocation(Core.workspace.getPlacementHint(ps));
      ps.show();
      return;
    }
    
    
    File f = new File(exportdir.getText());
    if (!exportdir.getText().equals("") && !(f.isDirectory() || f.canWrite())) {
      JOptionPane.showInternalMessageDialog(Core.workspace,Dict.get("gui.pathsettings.warn.message",f.getAbsolutePath()),Dict.get("gui.pathsettings.warn.title"),JOptionPane.WARNING_MESSAGE);
      return;
    }
    
    f = new File(elementdir.getText());
    if (!elementdir.getText().equals("") && !(f.isDirectory() || f.canWrite())) {
      JOptionPane.showInternalMessageDialog(Core.workspace,Dict.get("gui.pathsettings.warn.message",f.getAbsolutePath()),Dict.get("gui.pathsettings.warn.title"),JOptionPane.WARNING_MESSAGE);
      return;
    }
    
    Preferences cfg = Preferences.userNodeForPackage(Core.class);
    String val = nenyadir.getText();
    
    if (val.equals("")) cfg.remove("nenyabase");
    else cfg.put("nenyabase",val);
    
    val = exportdir.getText();
    if (val.equals("")) cfg.remove("centralexportdir");
    else cfg.put("centralexportdir",val);
    
    val = elementdir.getText();
    if (val.equals("")) cfg.remove("centralelementdir");
    else cfg.put("centralelementdir",val);
    
    // This will reload the elementlist in the ProjectEditor
    Core.workspace.setProject(Core.workspace.getProject());
    JOptionPane.showInternalMessageDialog(Core.workspace,Dict.get("gui.pathsettings.saved.message"),Dict.get("gui.pathsettings.saved.title"),JOptionPane.INFORMATION_MESSAGE);
    
    
  }
   
  /**
   * Call this to get a Hook to be placed into the config menu
   */
  public static JMenuItem getHook() {
    if (item!=null) return item;
    PathSettings ps = new PathSettings(0);
    item = new JMenuItem(Dict.get("gui.pathsettings.menuitem"));
    item.addActionListener(new PathSettings(0));
        
    return item;
  }
}