package de.opalpiraten.scenepainter.gui.render;
import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.geom.*;
import de.opalpiraten.scenepainter.sys.*;
import com.threerings.media.image.*;
import com.threerings.media.tile.*;


/**
 * A SceneElement, that provides a nenya media (com.threerings.media.tile)
 */
public class TileSceneElement extends SceneElement {

  /**
   * Resource Identifier (nenya knows how to handle it)
   */
  private String resource;
  
  /**
   * Tileindex - which of the possible several images to show
   */
  private int index;
  
  /**
   * Colorizations to apply to this Tile. Key is the name of a colorclass,
   * value the name of a colorrecord.
   */
  private Hashtable zations;
  
  /**
   * Set TRI of the resource to show
   * @param id the pathname of the tile
   */
  public void setResource(String id) {
    resource=id;
    String tmp[] = id.split("/");
    if (tmp.length>0) setName(tmp[tmp.length-1]);
    else setName("???");
    cache=null;
  }
  
  /**
   * Query resource resourceidentifier
   */
  public String getResource() {
    return resource;
  }
  
  
  /**
   * Submit colorizations to supply.
   * @param c Expected is a hashtable, that maps the names of color classes
   * to the name of colors. Every key/value pair is used to retrieve one
   * colorization from the colorpository and apply it to the tileset.
   */
  public void setZations(Hashtable c) {
    zations=c;
    cache=null;
  }
  
  /**
   * Query the colorisations of this tile
   */
  public Hashtable getZations() {
    return zations;
  }
  
  /**
   * Set Index (the number of the image) to show
   */
  public void setIndex(int i) {
    index=i;
    cache=null;
  }
  
  /**
   * Query the Index of the showing image
   */
  public int getIndex() {
    return index;
  }
  
  // Overridden
  public void scrollView(boolean b) {
    try {
      TileSet tset=Core.nenyabrowser.getTileManager().getTileSet(resource);
      if (b) index++;
      else index--;
      if (index<0) index=tset.getTileCount()-1;
      if (index!=0) index=index % (tset.getTileCount());
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    cache=null;
  }
  
  // Abstract method implementation
  public BufferedImage buildCache() {
    BufferedImage ret = null;
    try {
      TileSet tset=Core.nenyabrowser.getTileManager().getTileSet(resource);
      Tile tile = tset.getTile(index, getColors());
      ret = new BufferedImage(tile.getWidth(),tile.getHeight(),BufferedImage.TYPE_INT_ARGB);
      Graphics2D g = ret.createGraphics();
      g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,alpha));
      tile.paint(g,0,0);
    }
    catch (Exception e) {
      // Somethings pretty wrong (probably a defect savefile). Build up
      // Something to show it.
      e.printStackTrace();
      ret=new BufferedImage(1,1,BufferedImage.TYPE_INT_ARGB);
      Graphics g = ret.createGraphics();
      FontMetrics fm = g.getFontMetrics();
      Rectangle2D bounds = fm.getStringBounds(resource,g);
      ret =new BufferedImage((int)(bounds.getWidth()+2),(int)(bounds.getHeight()+2),BufferedImage.TYPE_INT_ARGB);
      g = ret.createGraphics();
      g.setColor(Color.RED.brighter());
      g.fillRect(0,0,(int)(bounds.getWidth()+2),(int)(bounds.getHeight()+2) );
      g.setColor(Color.BLACK);
      g.drawString(resource,1,1+fm.getMaxAscent());
    }
    
    return ret;
  }
  
  /**
   * Unpack the zations Hashtable
   */
  private Colorization[] getColors() {
    Colorization ret[] = new Colorization[0];
    ColorPository  pos = project.getColorPository();
    if (zations!=null) {
      ret =new Colorization[zations.size()];
      Enumeration e = zations.keys();
      int i=0;
      while(e.hasMoreElements()) {
        String clazz = (String)e.nextElement();
        String cname = (String)zations.get(clazz);
        ret[i]=pos.getColorization(clazz,cname);
        i++;
      }
    }
    return ret;
  }

}